<?php

/**
 * Api observer model
 *
 * @category    Mage
 * @package     Mage_Api
 */
class Mage_Api_Model_Observer
{
    /**
     * @param Varien_Event_Observer $observer
     */
    public function apiUserAuthenticated($observer)
    {
        if (!Mage::getStoreConfigFlag(Mage_Api_Model_User::XML_PATH_API_CONFIG_ENABLE_BRUTE_FORCE_PROTECTION)) {
            return;
        }

        /** @var Mage_Api_Model_User $user */
        $user = $observer->getEvent()->getModel()->reload();

        if ($user->getId()) {
            $user->getResource()->resetFailedLogins($user);
        }
    }

    /**
     * @param Varien_Event_Observer $observer
     */
    public function apiUserAuthenticationFailed($observer)
    {
        if (!Mage::getStoreConfigFlag(Mage_Api_Model_User::XML_PATH_API_CONFIG_ENABLE_BRUTE_FORCE_PROTECTION)) {
            return;
        }

        /** @var Mage_Api_Model_User $user */
        $userName = $observer->getEvent()->getUserName();
        $user = Mage::getModel('api/user')->loadByUsername($userName)->reload();

        if ($user->getId()) {
            $user->getResource()->countUpFailedLogin($user);
        }
    }

    /**
     * @param Varien_Event_Observer $observer
     */
    public function apiUserSaveAfter($observer)
    {
        if (!Mage::getStoreConfigFlag(Mage_Api_Model_User::XML_PATH_API_CONFIG_ENABLE_BRUTE_FORCE_PROTECTION)) {
            return;
        }

        /** @var Mage_Api_Model_User $user */
        $user = $observer->getEvent()->getObject();

        if ($user->getData('api_key') !== $user->getOrigData('api_key')) {
            Mage::dispatchEvent('admin_user_changed_password', array(
                'user' => $user
            ));
        }
    }

    /**
     * @param Varien_Event_Observer $observer
     */
    public function apiUserChangedPassword($observer)
    {
        if (!Mage::getStoreConfigFlag(Mage_Api_Model_User::XML_PATH_API_CONFIG_ENABLE_BRUTE_FORCE_PROTECTION)) {
            return;
        }

        /** @var Mage_Api_Model_User $user */
        $user = $observer->getEvent()->getUser()->reload();

        if ($user->getId()) {
            $user->getResource()->resetFailedLogins($user);
        }
    }
}
