<?php

class Mage_Core_Model_Observer_Htaccess
{
    public function addMissingHtaccessFiles($observer)
    {
        switch (Mage::App()->getRequest()->getMethod()) {
            case 'POST':
            case 'PUT':
            case 'DELETE':
                foreach ($this->getPathsForDenyFromAll() as $pathForDenyFromAll) {
                    if (!is_dir(dirname($pathForDenyFromAll))) {
                        continue;
                    }

                    if (!file_exists($pathForDenyFromAll)) {
                        if (is_writeable(dirname($pathForDenyFromAll))) {
                            Mage::log($pathForDenyFromAll . ' does not exist, trying to reset');
                            file_put_contents($pathForDenyFromAll, $this->getHtaccessDenyFromAllContent());
                        } else {
                            Mage::log($pathForDenyFromAll . ' does not exist and file is not writeable');
                        }
                    }
                }

                $mediaHtaccessPath = Mage::getBaseDir('media') . '/.htaccess';

                if (!file_exists($mediaHtaccessPath)) {
                    if (is_writeable(dirname($mediaHtaccessPath))) {
                        Mage::log($mediaHtaccessPath . ' does not exist, trying to reset');
                        file_put_contents($mediaHtaccessPath, $this->getMediaHtaccessContent());
                    } else {
                        Mage::log($mediaHtaccessPath . ' does not exist and file is not writeable');
                    }
                }
        }
    }

    protected function getPathsForDenyFromAll()
    {
        return [
            Mage::getBaseDir('app') . '/.htaccess',
            Mage::getBaseDir('base') . '/dev/.htaccess',
            Mage::getBaseDir('base') . '/downloader/lib/.htaccess',
            Mage::getBaseDir('base') . '/downloader/Maged/.htaccess',
            Mage::getBaseDir('base') . '/downloader/template/.htaccess',
            Mage::getBaseDir('base') . '/includes/.htaccess',
            Mage::getBaseDir('lib') . '/.htaccess',
            Mage::getBaseDir('media') . '/downloadable/.htaccess',
            Mage::getBaseDir('media') . '/customer/.htaccess',
            Mage::getBaseDir('base') . '/shell/.htaccess',
            Mage::getBaseDir('skin') . '/frontend/rwd/default/scss/.htaccess',
            Mage::getBaseDir('var') . '/.htaccess',
        ];
    }

    protected function getHtaccessDenyFromAllContent()
    {
        return <<<EOD
Order deny,allow
Deny from all

EOD;
    }

    protected function getMediaHtaccessContent()
    {
        return <<<EOD
Options All -Indexes
<IfModule mod_php5.c>
php_flag engine 0
</IfModule>

AddHandler cgi-script .php .pl .py .jsp .asp .htm .shtml .sh .cgi
Options -ExecCGI

<IfModule mod_rewrite.c>

############################################
## enable rewrites

    Options +FollowSymLinks
    RewriteEngine on

############################################
## never rewrite for existing files
    RewriteCond %{REQUEST_FILENAME} !-f

############################################
## rewrite everything else to index.php

    RewriteRule .* ../get.php [L]
</IfModule>

EOD;
    }
}
